@@ -3,10 +3,20 @@ use strict;
 use warnings;
 
 use Carp;
-use WWW::Mechanize;
+#use WWW::Mechanize;
 use Net::Whois::RIPE;
 use Data::Dumper;
 
+our $MECHANIZED;
+BEGIN {
+    $MECHANIZED    = do {
+        eval {
+            require WWW::Mechanize;
+        };
+        ($@) ? 0 : 1;
+    };
+}
+
 =head1 NAME
 
 Net::Whois::Object - Object encapsulating RPSL data returned by Whois queries
@@ -72,6 +82,8 @@ RIPE provides several web interfaces
 Although not the latest one, this simple interface is the first to be wrapped
 by this module.
 
+B<CAUTION: SyncUpdates features require WWW::Mechanize to be installed.>
+
 =head4 Create
 
 Once the object has been modified, locally, you can create it in the database
@@ -549,6 +561,8 @@ Return the HTML code of the returned page.
 sub _syncupdates_submit {
     my ( $self, $text, $password ) = @_;
 
+    croak "WWW::Mechanize required for updates"     unless $MECHANIZED;    
+
     $text .= "password: $password\n" if $password;
 
     my $mech = WWW::Mechanize->new();
