@@ -48,6 +48,12 @@ sub new {
         $self->$key( $options{$key} );
     }
 
+    $self->attributes( 'primary', ['as_block'] );
+    $self->attributes( 'mandatory', [ 'as_block', 'admin_c', 'tech_c', 'mnt_by', 'changed', 'source' ] );
+    $self->attributes( 'optionnal', [ 'descr', 'remarks', 'org', 'notify', 'mnt_lower' ] );
+    $self->attributes( 'single', [ 'as_block', 'source' ] );
+    $self->attributes( 'multiple', [ 'descr', 'remarks', 'org', 'admin_c', 'tech_c', 'notify', 'mnt_lower', 'mnt_by', 'changed' ] );
+
     return $self;
 }
 
@@ -68,8 +74,8 @@ stored in the appropriate Internet Registry's Whois Database.
 
 sub as_block {
     my ( $self, $as_block ) = @_;
-    $self->{as_block} = $as_block if defined $as_block;
-    return $self->{as_block};
+
+    return $self->_single_attribute_setget( 'as_block', $as_block );
 }
 
 =head2 B<descr( [$descr] )>
@@ -84,8 +90,8 @@ in the as-block.
 
 sub descr {
     my ( $self, $descr ) = @_;
-    push @{ $self->{descr} }, $descr if defined $descr;
-    return \@{ $self->{descr} };
+
+    return $self->_multiple_attribute_setget( 'descr', $descr );
 }
 
 =head2 B<remarks( [$remarks] )>
@@ -104,8 +110,8 @@ the AS numbers.
 
 sub remarks {
     my ( $self, $remarks ) = @_;
-    push @{ $self->{remarks} }, $remarks if defined $remarks;
-    return \@{ $self->{remarks} };
+
+    return $self->_multiple_attribute_setget( 'remarks', $remarks );
 }
 
 =head2 B<tech_c( [$tech_c] )>
@@ -127,8 +133,8 @@ physically located at the site of the network.
 
 sub tech_c {
     my ( $self, $tech_c ) = @_;
-    push @{ $self->{tech_c} }, $tech_c if defined $tech_c;
-    return \@{ $self->{tech_c} };
+
+    return $self->_multiple_attribute_setget( 'tech_c', $tech_c );
 }
 
 =head2 B<admin_c( [$admin_c])>
@@ -147,8 +153,8 @@ located at the site of the network.
 
 sub admin_c {
     my ( $self, $admin_c ) = @_;
-    push @{ $self->{admin_c} }, $admin_c if defined $admin_c;
-    return \@{ $self->{admin_c} };
+
+    return $self->_multiple_attribute_setget( 'admin_c', $admin_c );
 }
 
 =head2 B<notify( [$notify] )>
@@ -164,8 +170,8 @@ to the object should be sent.
 
 sub notify {
     my ( $self, $notify ) = @_;
-    push @{ $self->{notify} }, $notify if defined $notify;
-    return \@{ $self->{notify} };
+
+    return $self->_multiple_attribute_setget( 'notify', $notify );
 }
 
 =head2 B<mnt_lower( [$mnt_lower] )>
@@ -184,8 +190,8 @@ authorization.
 
 sub mnt_lower {
     my ( $self, $mnt_lower ) = @_;
-    push @{ $self->{mnt_lower} }, $mnt_lower if defined $mnt_lower;
-    return \@{ $self->{mnt_lower} };
+
+    return $self->_multiple_attribute_setget( 'mnt_lower', $mnt_lower );
 }
 
 =head2 B<mnt_by( [$mnt_by] )>
@@ -205,8 +211,8 @@ object will be able to change details.
 
 sub mnt_by {
     my ( $self, $mnt_by ) = @_;
-    push @{ $self->{mnt_by} }, $mnt_by if defined $mnt_by;
-    return \@{ $self->{mnt_by} };
+
+    return $self->_multiple_attribute_setget( 'mnt_by', $mnt_by );
 }
 
 =head2 B<changed( [$changed] )>
@@ -228,8 +234,8 @@ format using one of the following two formats: YYYYMMDD or YYMMDD.
 
 sub changed {
     my ( $self, $changed ) = @_;
-    push @{ $self->{changed} }, $changed if defined $changed;
-    return \@{ $self->{changed} };
+
+    return $self->_multiple_attribute_setget( 'changed', $changed );
 }
 
 =head2 B<source( [$source] )>
@@ -243,8 +249,8 @@ The database where the object is registered.
 
 sub source {
     my ( $self, $source ) = @_;
-    $self->{source} = $source if defined $source;
-    return $self->{source};
+
+    return $self->_single_attribute_setget( 'source', $source );
 }
 
 =head2 B<org( [$org] )>
@@ -258,8 +264,8 @@ The organisation entity this object is bound to.
 
 sub org {
     my ( $self, $org ) = @_;
-    $self->{org} = $org if defined $org;
-    return $self->{org};
+
+    return $self->_single_attribute_setget( 'org', $org );
 }
 
 1;
