@@ -46,6 +46,12 @@ sub new {
         $self->$key( $options{$key} );
     }
 
+    $self->attributes( 'primary',   ['filter_set'] );
+    $self->attributes( 'mandatory', [ 'filter_set', 'filter', 'mp_filter', 'source' ] );
+    $self->attributes( 'optionnal', [ 'remarks', 'org', 'notify', 'mnt_lower' ] );
+    $self->attributes( 'single',    [ 'filter_set', 'filter', 'mp_filter', 'source' ] );
+    $self->attributes( 'multiple',  [ 'descr', 'remarks', 'org', 'tech_c', 'admin_c', 'notify', 'mnt_by', 'mnt_lower', 'changed' ] );
+
     return $self;
 }
 
@@ -68,12 +74,11 @@ components of a hierarchical filter-name have to be filter_set names.
 
 sub filter_set {
     my ( $self, $filter_set ) = @_;
-    if ($filter_set and $filter_set !~/^fltr-/i) {
-        warn "Incorrect FilterSet's name ($filter_set) : Should start with
-        'FLTR-'";
+    if ( $filter_set and $filter_set !~ /^fltr-/i ) {
+        warn "Incorrect FilterSet's name ($filter_set) : Should start with 'FLTR-'";
     }
-    $self->{filter_set} = $filter_set if defined $filter_set;
-    return $self->{filter_set};
+
+    return $self->_single_attribute_setget( 'filter_set', $filter_set );
 }
 
 =head2 B<descr( [$descr] )>
@@ -88,8 +93,8 @@ A short description related to the object's purpose.
 
 sub descr {
     my ( $self, $descr ) = @_;
-    push @{ $self->{descr} }, $descr if defined $descr;
-    return \@{ $self->{descr} };
+
+    return $self->_multiple_attribute_setget( 'descr', $descr );
 }
 
 =head2 B<filter( [$filter] )>
@@ -107,8 +112,8 @@ you have said you want to see.
 
 sub filter {
     my ( $self, $filter ) = @_;
-    $self->{filter} = $filter if defined $filter;
-    return $self->{filter};
+
+    return $self->_single_attribute_setget( 'filter', $filter );
 }
 
 =head2 B<mp_filter( [$mp_filter] )>
@@ -123,8 +128,8 @@ a subset of these routes.
 
 sub mp_filter {
     my ( $self, $mp_filter ) = @_;
-    $self->{mp_filter} = $mp_filter if defined $mp_filter;
-    return $self->{mp_filter};
+
+    return $self->_single_attribute_setget( 'mp_filter', $mp_filter );
 }
 
 =head2 B<remarks( [$remark] )>
@@ -139,8 +144,8 @@ General remarks. May include a URL or email address.
 
 sub remarks {
     my ( $self, $remark ) = @_;
-    push @{ $self->{remarks} }, $remark if defined $remark;
-    return \@{ $self->{remarks} };
+
+    return $self->_multiple_attribute_setget( 'remarks', $remark );
 }
 
 =head2 B<tech_c( [$contact] )>
@@ -161,8 +166,8 @@ physically located at the site of the network.
 
 sub tech_c {
     my ( $self, $contact ) = @_;
-    push @{ $self->{tech_c} }, $contact if defined $contact;
-    return \@{ $self->{tech_c} };
+
+    return $self->_multiple_attribute_setget( 'tech_c', $contact );
 }
 
 =head2 B<admin_c( [$contact] )>
@@ -181,8 +186,8 @@ located at the site of the network.
 
 sub admin_c {
     my ( $self, $contact ) = @_;
-    push @{ $self->{admin_c} }, $contact if defined $contact;
-    return \@{ $self->{admin_c} };
+
+    return $self->_multiple_attribute_setget( 'admin_c', $contact );
 }
 
 =head2 B<org( [$org] )>
@@ -197,8 +202,8 @@ The organisation responsible for this FilterSet object.
 
 sub org {
     my ( $self, $org ) = @_;
-    push @{ $self->{org} }, $org if defined $org;
-    return \@{ $self->{org} };
+
+    return $self->_multiple_attribute_setget( 'org', $org );
 }
 
 =head2 B<notify( [$notify] )>
@@ -220,8 +225,8 @@ format using one of the following two formats: YYYYMMDD or YYMMDD.
 
 sub notify {
     my ( $self, $notify ) = @_;
-    push @{ $self->{notify} }, $notify if defined $notify;
-    return \@{ $self->{notify} };
+
+    return $self->_multiple_attribute_setget( 'notify', $notify );
 }
 
 =head2 B<mnt_by( [$mnt_by] )>
@@ -241,8 +246,8 @@ object will be able to change details.
 
 sub mnt_by {
     my ( $self, $mnt_by ) = @_;
-    push @{ $self->{mnt_by} }, $mnt_by if defined $mnt_by;
-    return \@{ $self->{mnt_by} };
+
+    return $self->_multiple_attribute_setget( 'mnt_by', $mnt_by );
 }
 
 =head2 B<mnt_lower( [$mnt_lower] )>
@@ -258,8 +263,8 @@ used as well as mnt_by.
 
 sub mnt_lower {
     my ( $self, $mnt_lower ) = @_;
-    push @{ $self->{mnt_lower} }, $mnt_lower if defined $mnt_lower;
-    return \@{ $self->{mnt_lower} };
+
+    return $self->_multiple_attribute_setget( 'mnt_lower', $mnt_lower );
 }
 
 =head2 B<changed( [$changed] )>
@@ -281,8 +286,8 @@ format using one of the following two formats: YYYYMMDD or YYMMDD.
 
 sub changed {
     my ( $self, $changed ) = @_;
-    push @{ $self->{changed} }, $changed if defined $changed;
-    return \@{ $self->{changed} };
+
+    return $self->_multiple_attribute_setget( 'changed', $changed );
 }
 
 =head2 B<source( [$source] )>
@@ -296,8 +301,8 @@ The database where the object is registered.
 
 sub source {
     my ( $self, $source ) = @_;
-    $self->{source} = $source if defined $source;
-    return $self->{source};
+
+    return $self->_single_attribute_setget( 'source', $source );
 }
 
 1;
