@@ -53,6 +53,12 @@ sub new {
         $self->$key( $options{$key} );
     }
 
+    $self->attributes( 'primary',   ['inet6num'] );
+    $self->attributes( 'mandatory', [ 'inet6num', 'netname', 'status', 'source' ] );
+    $self->attributes( 'optionnal', [ 'org', 'remarks', 'notify', 'mnt_lower', 'mnt_routes', 'mnt_domains', 'mnt_irt' ] );
+    $self->attributes( 'single', [ 'inet6num', 'netname', 'org', 'status', 'source' ] );
+    $self->attributes( 'multiple', [ 'descr', 'country', 'tech_c', 'admin_c', 'remarks', 'notify', 'mnt_by', 'mnt_lower', 'mnt_routes', 'mnt_domains', 'mnt_irt', 'changed' ] );
+
     return $self;
 }
 
@@ -70,8 +76,8 @@ Addresses can only be expressed in prefix notation
 
 sub inet6num {
     my ( $self, $inet6num ) = @_;
-    $self->{inet6num} = $inet6num if defined $inet6num;
-    return $self->{inet6num};
+
+    return $self->_single_attribute_setget( 'inet6num', $inet6num );
 }
 
 =head2 B<netname( [$netname] )>
@@ -87,8 +93,8 @@ used for a common purpose.
 
 sub netname {
     my ( $self, $netname ) = @_;
-    $self->{netname} = $netname if defined $netname;
-    return $self->{netname};
+
+    return $self->_single_attribute_setget( 'netname', $netname );
 }
 
 =head2 B<descr( [$descr] )>
@@ -104,8 +110,8 @@ in the inet6num.
 
 sub descr {
     my ( $self, $descr ) = @_;
-    push @{ $self->{descr} }, $descr if defined $descr;
-    return \@{ $self->{descr} };
+
+    return $self->_multiple_attribute_setget( 'descr', $descr );
 }
 
 =head2 B<country( [$country] )>
@@ -124,8 +130,8 @@ map IP addresses to countries.
 
 sub country {
     my ( $self, $country ) = @_;
-    push @{ $self->{country} }, $country if defined $country;
-    return \@{ $self->{country} };
+
+    return $self->_multiple_attribute_setget( 'country', $country );
 }
 
 =head2 B<org( [$org] )>
@@ -141,8 +147,8 @@ resource.
 
 sub org {
     my ( $self, $org ) = @_;
-    $self->{org} = $org if defined $org;
-    return $self->{org};
+
+    return $self->_single_attribute_setget( 'org', $org );
 }
 
 =head2 B<admin_c( [$contact] )>
@@ -161,8 +167,8 @@ located at the site of the network.
 
 sub admin_c {
     my ( $self, $contact ) = @_;
-    push @{ $self->{admin_c} }, $contact if defined $contact;
-    return \@{ $self->{admin_c} };
+
+    return $self->_multiple_attribute_setget( 'admin_c', $contact );
 }
 
 =head2 B<tech_c( [$contact] )>
@@ -181,9 +187,9 @@ day-to-day operation of the network, but does not need to be
 =cut
 
 sub tech_c {
-    my ( $self, $tech_c ) = @_;
-    push @{ $self->{tech_c} }, $tech_c if defined $tech_c;
-    return \@{ $self->{tech_c} };
+    my ( $self, $contact ) = @_;
+
+    return $self->_multiple_attribute_setget( 'tech_c', $contact );
 }
 
 =head2 B<status( [$status] )>
@@ -214,8 +220,8 @@ Status can have one of these values:
 
 sub status {
     my ( $self, $status ) = @_;
-    $self->{status} = $status if defined $status;
-    return $self->{status};
+
+    return $self->_single_attribute_setget( 'status', $status );
 }
 
 =head2 B<remarks( [$remark] )>
@@ -230,9 +236,9 @@ complaints.
 =cut
 
 sub remarks {
-    my ( $self, $remarks ) = @_;
-    push @{ $self->{remarks} }, $remarks if defined $remarks;
-    return \@{ $self->{remarks} };
+    my ( $self, $remark ) = @_;
+
+    return $self->_multiple_attribute_setget( 'remarks', $remark );
 }
 
 =head2 B<notify( [$notify] )>
@@ -248,8 +254,8 @@ sent.
 
 sub notify {
     my ( $self, $notify ) = @_;
-    push @{ $self->{notify} }, $notify if defined $notify;
-    return \@{ $self->{notify} };
+
+    return $self->_multiple_attribute_setget( 'notify', $notify );
 }
 
 =head2 B<mnt_by( [$mnt_by] )>
@@ -262,8 +268,8 @@ always return the current mnt_by array.
 
 sub mnt_by {
     my ( $self, $mnt_by ) = @_;
-    push @{ $self->{mnt_by} }, $mnt_by if defined $mnt_by;
-    return \@{ $self->{mnt_by} };
+
+    return $self->_multiple_attribute_setget( 'mnt_by', $mnt_by );
 }
 
 =head2 B<mnt_lower( [$mnt_lower] )>
@@ -279,8 +285,8 @@ used as well as 'mnt-by.'
 
 sub mnt_lower {
     my ( $self, $mnt_lower ) = @_;
-    push @{ $self->{mnt_lower} }, $mnt_lower if defined $mnt_lower;
-    return \@{ $self->{mnt_lower} };
+
+    return $self->_multiple_attribute_setget( 'mnt_lower', $mnt_lower );
 }
 
 =head2 B<mnt_routes( [$mnt_route] )>
@@ -297,8 +303,8 @@ object.
 
 sub mnt_routes {
     my ( $self, $mnt_route ) = @_;
-    push @{ $self->{mnt_routes} }, $mnt_route if defined $mnt_route;
-    return \@{ $self->{mnt_routes} };
+
+    return $self->_multiple_attribute_setget( 'mnt_routes', $mnt_route );
 }
 
 =head2 B<mnt_domains( [$mnt_route] )>
@@ -315,8 +321,8 @@ object.
 
 sub mnt_domains {
     my ( $self, $mnt_route ) = @_;
-    push @{ $self->{mnt_domains} }, $mnt_route if defined $mnt_route;
-    return \@{ $self->{mnt_domains} };
+
+    return $self->_multiple_attribute_setget( 'mnt_domains', $mnt_route );
 }
 
 =head2 B<mnt_irt( [$mnt_irt] )>
@@ -332,8 +338,8 @@ object to be able to add this reference.
 
 sub mnt_irt {
     my ( $self, $mnt_irt ) = @_;
-    push @{ $self->{mnt_irt} }, $mnt_irt if defined $mnt_irt;
-    return \@{ $self->{mnt_irt} };
+
+    return $self->_multiple_attribute_setget( 'mnt_irt', $mnt_irt );
 }
 
 =head2 B<changed( [$changed] )>
@@ -355,8 +361,8 @@ format using one of the following two formats: YYYYMMDD or YYMMDD.
 
 sub changed {
     my ( $self, $changed ) = @_;
-    push @{ $self->{changed} }, $changed if defined $changed;
-    return \@{ $self->{changed} };
+
+    return $self->_multiple_attribute_setget( 'changed', $changed );
 }
 
 =head2 B<source( [$source] )>
@@ -370,8 +376,8 @@ The database where the object is registered.
 
 sub source {
     my ( $self, $source ) = @_;
-    $self->{source} = $source if defined $source;
-    return $self->{source};
+
+    return $self->_single_attribute_setget( 'source', $source );
 }
 
 1;
