@@ -48,6 +48,12 @@ sub new {
         $self->$key( $options{$key} );
     }
 
+    $self->attributes( 'primary', ['inet_rtr'] );
+    $self->attributes( 'mandatory', [ 'inet_rtr', 'descr', 'local_as', 'ifaddr', 'tech_c', 'admin_c', 'mnt_by', 'changed', 'source' ] );
+    $self->attributes( 'optionnal', [ 'alias', 'interface', 'peer', 'mp_peer', 'member_of', 'remarks', 'org', 'notify' ] );
+    $self->attributes( 'single', [ 'inet_rtr', 'local_as', 'source' ] );
+    $self->attributes( 'multiple', [ 'descr', 'remarks', 'alias', 'ifaddr', 'interface', 'peer', 'mp_peer', 'member_of', 'org', 'tech_c', 'admin_c', 'notify', 'mnt_by', 'changed' ] );
+
     return $self;
 }
 
@@ -63,8 +69,8 @@ dot.
 
 sub inet_rtr {
     my ( $self, $inet_rtr ) = @_;
-    $self->{inet_rtr} = $inet_rtr if defined $inet_rtr;
-    return $self->{inet_rtr};
+
+    return $self->_single_attribute_setget( 'inet_rtr', $inet_rtr );
 }
 
 =head2 B<descr( [$descr] )>
@@ -79,8 +85,8 @@ A short description related to the object's purpose.
 
 sub descr {
     my ( $self, $descr ) = @_;
-    push @{ $self->{descr} }, $descr if defined $descr;
-    return \@{ $self->{descr} };
+
+    return $self->_multiple_attribute_setget( 'descr', $descr );
 }
 
 =head2 B<alias( [$alias] )>
@@ -96,8 +102,8 @@ specified router.
 
 sub alias {
     my ( $self, $alias ) = @_;
-    push @{ $self->{alias} }, $alias if defined $alias;
-    return \@{ $self->{alias} };
+
+    return $self->_multiple_attribute_setget( 'alias', $alias );
 }
 
 =head2 B<local_as( [$local_as] )>
@@ -112,8 +118,8 @@ operates this router.
 
 sub local_as {
     my ( $self, $local_as ) = @_;
-    $self->{local_as} = $local_as if defined $local_as;
-    return $self->{local_as};
+
+    return $self->_single_attribute_setget( 'local_as', $local_as );
 }
 
 =head2 B<ifaddr( [$ifaddr] )>
@@ -130,8 +136,8 @@ interface.
 
 sub ifaddr {
     my ( $self, $ifaddr ) = @_;
-    push @{ $self->{ifaddr} }, $ifaddr if defined $ifaddr;
-    return \@{ $self->{ifaddr} };
+
+    return $self->_multiple_attribute_setget( 'ifaddr', $ifaddr );
 }
 
 =head2 B<peer( [$peer] )>
@@ -147,8 +153,8 @@ peering.
 
 sub peer {
     my ( $self, $peer ) = @_;
-    push @{ $self->{peer} }, $peer if defined $peer;
-    return \@{ $self->{peer} };
+
+    return $self->_multiple_attribute_setget( 'peer', $peer );
 }
 
 =head2 B<member_of( [$member_of] )>
@@ -165,8 +171,8 @@ respective mbrs-by-ref attribute in the referenced object.
 
 sub member_of {
     my ( $self, $member_of ) = @_;
-    push @{ $self->{member_of} }, $member_of if defined $member_of;
-    return \@{ $self->{member_of} };
+
+    return $self->_multiple_attribute_setget( 'member_of', $member_of );
 }
 
 =head2 B<remarks( [$remark] )>
@@ -182,8 +188,8 @@ complaints.
 
 sub remarks {
     my ( $self, $remark ) = @_;
-    push @{ $self->{remarks} }, $remark if defined $remark;
-    return \@{ $self->{remarks} };
+
+    return $self->_multiple_attribute_setget( 'remarks', $remark );
 }
 
 =head2 B<admin_c( [$contact] )>
@@ -202,8 +208,8 @@ located at the site of the network.
 
 sub admin_c {
     my ( $self, $contact ) = @_;
-    push @{ $self->{admin_c} }, $contact if defined $contact;
-    return \@{ $self->{admin_c} };
+
+    return $self->_multiple_attribute_setget( 'admin_c', $contact );
 }
 
 =head2 B<tech_c( [$contact] )>
@@ -224,8 +230,8 @@ physically located at the site of the network.
 
 sub tech_c {
     my ( $self, $contact ) = @_;
-    push @{ $self->{tech_c} }, $contact if defined $contact;
-    return \@{ $self->{tech_c} };
+
+    return $self->_multiple_attribute_setget( 'tech_c', $contact );
 }
 
 =head2 B<notify( [$notify] )>
@@ -241,8 +247,8 @@ be sent.
 
 sub notify {
     my ( $self, $notify ) = @_;
-    push @{ $self->{notify} }, $notify if defined $notify;
-    return \@{ $self->{notify} };
+
+    return $self->_multiple_attribute_setget( 'notify', $notify );
 }
 
 =head2 B<mnt_by( [$mnt_by] )>
@@ -258,8 +264,8 @@ object.
 
 sub mnt_by {
     my ( $self, $mnt_by ) = @_;
-    push @{ $self->{mnt_by} }, $mnt_by if defined $mnt_by;
-    return \@{ $self->{mnt_by} };
+
+    return $self->_multiple_attribute_setget( 'mnt_by', $mnt_by );
 }
 
 =head2 B<changed( [$changed] )>
@@ -281,8 +287,8 @@ format using one of the following two formats: YYYYMMDD or YYMMDD.
 
 sub changed {
     my ( $self, $changed ) = @_;
-    push @{ $self->{changed} }, $changed if defined $changed;
-    return \@{ $self->{changed} };
+
+    return $self->_multiple_attribute_setget( 'changed', $changed );
 }
 
 =head2 B<source( [$source] )>
@@ -296,8 +302,8 @@ The database where the object is registered.
 
 sub source {
     my ( $self, $source ) = @_;
-    $self->{source} = $source if defined $source;
-    return $self->{source};
+
+    return $self->_single_attribute_setget( 'source', $source );
 }
 
 =head2 B<mp_peer( [$peer] )>
@@ -316,8 +322,8 @@ The mp-peer attribute extends the peer attribute for IPv6 addresses.
 
 sub mp_peer {
     my ( $self, $mp_peer ) = @_;
-    push @{ $self->{mp_peer} }, $mp_peer if defined $mp_peer;
-    return \@{ $self->{mp_peer} };
+
+    return $self->_multiple_attribute_setget( 'mp_peer', $mp_peer );
 }
 
 =head2 B<interface( [$interface] )>
@@ -333,8 +339,24 @@ an Internet router, optional action and tunnel definition.
 
 sub interface {
     my ( $self, $interface ) = @_;
-    push @{ $self->{interface} }, $interface if defined $interface;
-    return \@{ $self->{interface} };
+
+    return $self->_multiple_attribute_setget( 'interface', $interface );
+}
+
+=head2 B<org( [$org] )>
+
+Accessor to the org attribute.
+Accepts an optional org to be added to the org array,
+always return the current org array.
+
+The organisation entity this object is bound to.
+
+=cut
+
+sub org {
+    my ( $self, $org ) = @_;
+
+    return $self->_multiple_attribute_setget( 'org', $org );
 }
 
 1;
