@@ -35,6 +35,11 @@ sub new {
         $self->$key( $options{$key} );
     }
 
+    $self->attributes( 'mandatory', ['response'] );
+    $self->attributes( 'optionnal', ['comment'] );
+    $self->attributes( 'single',    ['response'] );
+    $self->attributes( 'multiple',  ['comment'] );
+
     return $self;
 }
 
@@ -47,8 +52,21 @@ Accepts an optional response, always return the current response.
 
 sub response {
     my ( $self, $response ) = @_;
-    $self->{response} = $response if defined $response;
-    return $self->{response};
+
+    return $self->_single_attribute_setget( 'response', $response );
+}
+
+=head2 B<comment( [$comment] )>
+
+Accessor to the comment attribute.
+Accepts an optional comment, always return the current comment.
+
+=cut
+
+sub comment {
+    my ( $self, $comment ) = @_;
+
+    return $self->_multiple_attribute_setget( 'comment', $comment );
 }
 
 1;
